__version__ = '0.8.5'
__author__ = 'Robert James, Fiona Alder'
__author_email__ = 'robert.james.19@ucl.ac.uk'

from .utils import *
from .socket_thread import *
from .device_interface import *
from .calibration import *
